@@ -22,7 +22,7 @@
  * */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
 #include "php.h"
@@ -111,8 +111,8 @@ static zval *saproxy_read_dimension(zend_object *object, zval *offset, int type,
 		}
 		VariantInit(&v);
 
-		res = php_com_do_invoke(proxy->obj, Z_STRVAL(proxy->indices[0]),
-			   	Z_STRLEN(proxy->indices[0]), DISPATCH_METHOD|DISPATCH_PROPERTYGET, &v,
+		res = php_com_do_invoke(proxy->obj, Z_STR(proxy->indices[0]),
+				DISPATCH_METHOD|DISPATCH_PROPERTYGET, &v,
 			   	proxy->dimensions, args, 0);
 
 		efree(args);
@@ -228,8 +228,8 @@ static void saproxy_write_dimension(zend_object *object, zval *offset, zval *val
 			return;
 		}
 		VariantInit(&v);
-		if (SUCCESS == php_com_do_invoke(proxy->obj, Z_STRVAL(proxy->indices[0]),
-					Z_STRLEN(proxy->indices[0]), DISPATCH_PROPERTYPUT, &v, proxy->dimensions + 1,
+		if (SUCCESS == php_com_do_invoke(proxy->obj, Z_STR(proxy->indices[0]),
+					DISPATCH_PROPERTYPUT, &v, proxy->dimensions + 1,
 					args, 0)) {
 			VariantClear(&v);
 		}
@@ -332,12 +332,12 @@ static int saproxy_objects_compare(zval *object1, zval *object2)
 	return -1;
 }
 
-static int saproxy_object_cast(zend_object *readobj, zval *writeobj, int type)
+static zend_result saproxy_object_cast(zend_object *readobj, zval *writeobj, int type)
 {
 	return FAILURE;
 }
 
-static int saproxy_count_elements(zend_object *object, zend_long *count)
+static zend_result saproxy_count_elements(zend_object *object, zend_long *count)
 {
 	php_com_saproxy *proxy = (php_com_saproxy*) object;
 	LONG ubound, lbound;
@@ -415,7 +415,7 @@ zend_object_handlers php_com_saproxy_handlers = {
 	NULL,									/* get_properties_for */
 };
 
-int php_com_saproxy_create(zend_object *com_object, zval *proxy_out, zval *index)
+void php_com_saproxy_create(zend_object *com_object, zval *proxy_out, zval *index)
 {
 	php_com_saproxy *proxy, *rel = NULL;
 
@@ -440,10 +440,7 @@ int php_com_saproxy_create(zend_object *com_object, zval *proxy_out, zval *index
 	ZVAL_DUP(&proxy->indices[proxy->dimensions-1], index);
 
 	zend_object_std_init(&proxy->std, php_com_saproxy_class_entry);
-	proxy->std.handlers = &php_com_saproxy_handlers;
 	ZVAL_OBJ(proxy_out, &proxy->std);
-
-	return 1;
 }
 
 /* iterator */
@@ -458,7 +455,7 @@ static void saproxy_iter_dtor(zend_object_iterator *iter)
 	efree(I);
 }
 
-static int saproxy_iter_valid(zend_object_iterator *iter)
+static zend_result saproxy_iter_valid(zend_object_iterator *iter)
 {
 	php_com_saproxy_iter *I = (php_com_saproxy_iter*)Z_PTR(iter->data);
 
