@@ -5,63 +5,30 @@
 /** @not-serializable */
 class IntlTimeZone
 {
-    /**
-     * @var int
-     * @cvalue TimeZone::SHORT
-     */
-    public const DISPLAY_SHORT = UNKNOWN;
-    /**
-     * @var int
-     * @cvalue TimeZone::LONG
-     */
-    public const DISPLAY_LONG = UNKNOWN;
+    /** @cvalue TimeZone::SHORT */
+    public const int DISPLAY_SHORT = UNKNOWN;
+    /** @cvalue TimeZone::LONG */
+    public const int DISPLAY_LONG = UNKNOWN;
 
-    /**
-     * @var int
-     * @cvalue TimeZone::SHORT_GENERIC
-     */
-    public const DISPLAY_SHORT_GENERIC = UNKNOWN;
-    /**
-     * @var int
-     * @cvalue TimeZone::LONG_GENERIC
-     */
-    public const DISPLAY_LONG_GENERIC = UNKNOWN;
-    /**
-     * @var int
-     * @cvalue TimeZone::SHORT_GMT
-     */
-    public const DISPLAY_SHORT_GMT = UNKNOWN;
-    /**
-     * @var int
-     * @cvalue TimeZone::LONG_GMT
-     */
-    public const DISPLAY_LONG_GMT = UNKNOWN;
-    /**
-     * @var int
-     * @cvalue TimeZone::SHORT_COMMONLY_USED
-     */
-    public const DISPLAY_SHORT_COMMONLY_USED = UNKNOWN;
-    /**
-     * @var int
-     * @cvalue TimeZone::GENERIC_LOCATION
-     */
-    public const DISPLAY_GENERIC_LOCATION = UNKNOWN;
+    /** @cvalue TimeZone::SHORT_GENERIC */
+    public const int DISPLAY_SHORT_GENERIC = UNKNOWN;
+    /** @cvalue TimeZone::LONG_GENERIC */
+    public const int DISPLAY_LONG_GENERIC = UNKNOWN;
+    /** @cvalue TimeZone::SHORT_GMT */
+    public const int DISPLAY_SHORT_GMT = UNKNOWN;
+    /** @cvalue TimeZone::LONG_GMT */
+    public const int DISPLAY_LONG_GMT = UNKNOWN;
+    /** @cvalue TimeZone::SHORT_COMMONLY_USED */
+    public const int DISPLAY_SHORT_COMMONLY_USED = UNKNOWN;
+    /** @cvalue TimeZone::GENERIC_LOCATION */
+    public const int DISPLAY_GENERIC_LOCATION = UNKNOWN;
 
-    /**
-     * @var int
-     * @cvalue UCAL_ZONE_TYPE_ANY
-     */
-    public const TYPE_ANY = UNKNOWN;
-    /**
-     * @var int
-     * @cvalue UCAL_ZONE_TYPE_CANONICAL
-     */
-    public const TYPE_CANONICAL = UNKNOWN;
-    /**
-     * @var int
-     * @cvalue UCAL_ZONE_TYPE_CANONICAL_LOCATION
-     */
-    public const TYPE_CANONICAL_LOCATION = UNKNOWN;
+    /** @cvalue UCAL_ZONE_TYPE_ANY */
+    public const int TYPE_ANY = UNKNOWN;
+    /** @cvalue UCAL_ZONE_TYPE_CANONICAL */
+    public const int TYPE_CANONICAL = UNKNOWN;
+    /** @cvalue UCAL_ZONE_TYPE_CANONICAL_LOCATION */
+    public const int TYPE_CANONICAL_LOCATION = UNKNOWN;
 
     private function __construct() {}
 
@@ -145,6 +112,13 @@ class IntlTimeZone
      */
     public static function getGMT(): IntlTimeZone {}
 
+#if U_ICU_VERSION_MAJOR_NUM >= 74
+    /**
+     * @alias intltz_get_iana_id
+     */
+    public static function getIanaID(string $timezoneId): string|false {}
+#endif
+
     /**
      * @tentative-return-type
      * @alias intltz_get_id
