@@ -16,306 +16,174 @@
 */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
 #include "php.h"
 #include "php_xsl.h"
+#include <libxslt/variables.h>
 #include "ext/libxml/php_libxml.h"
+#include "ext/dom/namespace_compat.h"
 
-/* {{{ php_xsl_xslt_string_to_xpathexpr()
-   Translates a string to a XPath Expression */
-static char *php_xsl_xslt_string_to_xpathexpr(const char *str)
+
+static zend_result php_xsl_xslt_apply_params(xsltTransformContextPtr ctxt, HashTable *params)
 {
-	const xmlChar *string = (const xmlChar *)str;
-
-	xmlChar *value;
-	int str_len;
-
-	str_len = xmlStrlen(string) + 3;
-
-	if (xmlStrchr(string, '"')) {
-		if (xmlStrchr(string, '\'')) {
-			php_error_docref(NULL, E_WARNING, "Cannot create XPath expression (string contains both quote and double-quotes)");
-			return NULL;
-		}
-		value = (xmlChar*) safe_emalloc (str_len, sizeof(xmlChar), 0);
-		snprintf((char*)value, str_len, "'%s'", string);
-	} else {
-		value = (xmlChar*) safe_emalloc (str_len, sizeof(xmlChar), 0);
-		snprintf((char *)value, str_len, "\"%s\"", string);
-	}
-	return (char *) value;
-}
-/* }}} */
-
-/* {{{ php_xsl_xslt_make_params()
-   Translates a PHP array to a libxslt parameters array */
-static char **php_xsl_xslt_make_params(HashTable *parht, int xpath_params)
-{
-
-	int parsize;
-	zval *value;
-	char *xpath_expr;
 	zend_string *string_key;
-	char **params = NULL;
-	int i = 0;
+	zval *value;
 
-	parsize = (2 * zend_hash_num_elements(parht) + 1) * sizeof(char *);
-	params = (char **)safe_emalloc((2 * zend_hash_num_elements(parht) + 1), sizeof(char *), 0);
-	memset((char *)params, 0, parsize);
-
-	ZEND_HASH_MAP_FOREACH_STR_KEY_VAL(parht, string_key, value) {
+	ZEND_HASH_MAP_FOREACH_STR_KEY_VAL(params, string_key, value) {
 		ZEND_ASSERT(string_key != NULL);
-		if (Z_TYPE_P(value) != IS_STRING) {
-			if (!try_convert_to_string(value)) {
-				efree(params);
-				return NULL;
-			}
-		}
+		/* Already a string because of setParameter() */
+		ZEND_ASSERT(Z_TYPE_P(value) == IS_STRING);
 
-		if (!xpath_params) {
-			xpath_expr = php_xsl_xslt_string_to_xpathexpr(Z_STRVAL_P(value));
-		} else {
-			xpath_expr = estrndup(Z_STRVAL_P(value), Z_STRLEN_P(value));
-		}
-		if (xpath_expr) {
-			params[i++] = estrndup(ZSTR_VAL(string_key), ZSTR_LEN(string_key));
-			params[i++] = xpath_expr;
+		int result = xsltQuoteOneUserParam(ctxt, (const xmlChar *) ZSTR_VAL(string_key), (const xmlChar *) Z_STRVAL_P(value));
+		if (result < 0) {
+			php_error_docref(NULL, E_WARNING, "Could not apply parameter \"%s\"", ZSTR_VAL(string_key));
+			return FAILURE;
 		}
 	} ZEND_HASH_FOREACH_END();
 
-	params[i++] = NULL;
-
-	return params;
+	return SUCCESS;
 }
-/* }}} */
 
-static void xsl_ext_function_php(xmlXPathParserContextPtr ctxt, int nargs, int type) /* {{{ */
+static void xsl_proxy_factory(xmlNodePtr node, zval *child, dom_object *intern, xmlXPathParserContextPtr ctxt)
 {
-	xsltTransformContextPtr tctxt;
-	zval *args = NULL;
-	zval retval;
-	int i;
-	int error = 0;
-	zend_fcall_info fci;
-	zval handler;
-	xmlXPathObjectPtr obj;
-	char *str;
-	xsl_object *intern;
-	zend_string *callable = NULL;
+	ZEND_ASSERT(node->type != XML_NAMESPACE_DECL);
 
+	/**
+	 * Upon freeing libxslt's context, every document that is not the *main* document will be freed by libxslt.
+	 * If a node of a document that is *not the main* document gets returned to userland, we'd free the node twice:
+	 * first by the cleanup of the xslt context, and then by our own refcounting mechanism.
+	 * To prevent this, we'll take a copy if the node is not from the main document.
+	 * It is important that we do not copy the node unconditionally, because that means that:
+	 *  - modifications to the node will only modify the copy, and not the original
+	 *  - accesses to the parent, path, ... will not work
+	 */
+	xsltTransformContextPtr transform_ctxt = (xsltTransformContextPtr) ctxt->context->extra;
+	if (node->doc != transform_ctxt->document->doc) {
+		node = xmlDocCopyNode(node, intern->document->ptr, 1);
+	}
+	php_dom_create_object(node, child, intern);
+}
 
-	if (! zend_is_executing()) {
+static xsl_object *xsl_ext_fetch_intern(xmlXPathParserContextPtr ctxt)
+{
+	if (UNEXPECTED(!zend_is_executing())) {
 		xsltGenericError(xsltGenericErrorContext,
 		"xsltExtFunctionTest: Function called from outside of PHP\n");
-		error = 1;
+		return NULL;
+	}
+
+	xsltTransformContextPtr tctxt = xsltXPathGetTransformContext(ctxt);
+	if (UNEXPECTED(tctxt == NULL)) {
+		xsltGenericError(xsltGenericErrorContext,
+		"xsltExtFunctionTest: failed to get the transformation context\n");
+		return NULL;
+	}
+
+	xsl_object *intern = (xsl_object *) tctxt->_private;
+	if (UNEXPECTED(intern == NULL)) {
+		xsltGenericError(xsltGenericErrorContext,
+		"xsltExtFunctionTest: failed to get the internal object\n");
+		return NULL;
+	}
+	return intern;
+}
+
+static void xsl_ext_function_php(xmlXPathParserContextPtr ctxt, int nargs, php_dom_xpath_nodeset_evaluation_mode evaluation_mode) /* {{{ */
+{
+	xsl_object *intern = xsl_ext_fetch_intern(ctxt);
+	if (!intern) {
+		php_dom_xpath_callbacks_clean_argument_stack(ctxt, nargs);
 	} else {
-		tctxt = xsltXPathGetTransformContext(ctxt);
-		if (tctxt == NULL) {
-			xsltGenericError(xsltGenericErrorContext,
-			"xsltExtFunctionTest: failed to get the transformation context\n");
-			error = 1;
-		} else {
-			intern = (xsl_object*)tctxt->_private;
-			if (intern == NULL) {
-				xsltGenericError(xsltGenericErrorContext,
-				"xsltExtFunctionTest: failed to get the internal object\n");
-				error = 1;
-			}
-			else if (intern->registerPhpFunctions == 0) {
-				xsltGenericError(xsltGenericErrorContext,
-				"xsltExtFunctionTest: PHP Object did not register PHP functions\n");
-				error = 1;
-			}
-		}
-	}
-
-	if (error == 1) {
-		for (i = nargs - 1; i >= 0; i--) {
-			obj = valuePop(ctxt);
-			if (obj) {
-				xmlXPathFreeObject(obj);
-			}
-		}
-		return;
-	}
-
-	if (UNEXPECTED(nargs == 0)) {
-		zend_throw_error(NULL, "Function name must be passed as the first argument");
-		return;
-	}
-
-	fci.param_count = nargs - 1;
-	if (fci.param_count > 0) {
-		args = safe_emalloc(fci.param_count, sizeof(zval), 0);
-	}
-	/* Reverse order to pop values off ctxt stack */
-	for (i = fci.param_count - 1; i >= 0; i--) {
-		obj = valuePop(ctxt);
-		if (obj == NULL) {
-			ZVAL_NULL(&args[i]);
-			continue;
-		}
-		switch (obj->type) {
-			case XPATH_STRING:
-				ZVAL_STRING(&args[i], (char *)obj->stringval);
-				break;
-			case XPATH_BOOLEAN:
-				ZVAL_BOOL(&args[i],  obj->boolval);
-				break;
-			case XPATH_NUMBER:
-				ZVAL_DOUBLE(&args[i], obj->floatval);
-				break;
-			case XPATH_NODESET:
-				if (type == 1) {
-					str = (char*)xmlXPathCastToString(obj);
-					ZVAL_STRING(&args[i], str);
-					xmlFree(str);
-				} else if (type == 2) {
-					int j;
-					dom_object *domintern = (dom_object *)intern->doc;
-					if (obj->nodesetval && obj->nodesetval->nodeNr > 0) {
-						array_init(&args[i]);
-						for (j = 0; j < obj->nodesetval->nodeNr; j++) {
-							xmlNodePtr node = obj->nodesetval->nodeTab[j];
-							zval child;
-							/* not sure, if we need this... it's copied from xpath.c */
-							if (node->type == XML_NAMESPACE_DECL) {
-								xmlNsPtr curns;
-								xmlNodePtr nsparent;
-
-								nsparent = node->_private;
-								curns = xmlNewNs(NULL, node->name, NULL);
-								if (node->children) {
-									curns->prefix = xmlStrdup((xmlChar *)node->children);
-								}
-								if (node->children) {
-									node = xmlNewDocNode(node->doc, NULL, (xmlChar *) node->children, node->name);
-								} else {
-									node = xmlNewDocNode(node->doc, NULL, (const xmlChar *) "xmlns", node->name);
-								}
-								node->type = XML_NAMESPACE_DECL;
-								node->parent = nsparent;
-								node->ns = curns;
-							} else {
-								/**
-								 * Upon freeing libxslt's context, every document which is not the *main* document will be freed by libxslt.
-								 * If a node of a document which is *not the main* document gets returned to userland, we'd free the node twice:
-								 * first by the cleanup of the xslt context, and then by our own refcounting mechanism.
-								 * To prevent this, we'll take a copy if the node is not from the main document.
-								 * It is important that we do not copy the node unconditionally, because that means that:
-								 *  - modifications to the node will only modify the copy, and not the original
-								 *  - accesses to the parent, path, ... will not work
-								 */
-								xsltTransformContextPtr transform_ctxt = (xsltTransformContextPtr) ctxt->context->extra;
-								if (node->doc != transform_ctxt->document->doc) {
-									node = xmlDocCopyNode(node, domintern->document->ptr, 1);
-								}
-							}
-
-							php_dom_create_object(node, &child, domintern);
-							add_next_index_zval(&args[i], &child);
-						}
-					} else {
-						ZVAL_EMPTY_ARRAY(&args[i]);
-					}
-				}
-				break;
-			default:
-				str = (char *) xmlXPathCastToString(obj);
-				ZVAL_STRING(&args[i], str);
-				xmlFree(str);
-		}
-		xmlXPathFreeObject(obj);
-	}
-
-	fci.size = sizeof(fci);
-	fci.named_params = NULL;
-	if (fci.param_count > 0) {
-		fci.params = args;
-	} else {
-		fci.params = NULL;
-	}
-
-	/* Last element of the stack is the function name */
-	obj = valuePop(ctxt);
-	if (obj == NULL || obj->stringval == NULL) {
-		php_error_docref(NULL, E_WARNING, "Handler name must be a string");
-		xmlXPathFreeObject(obj);
-		valuePush(ctxt, xmlXPathNewString((const xmlChar *) ""));
-		if (fci.param_count > 0) {
-			for (i = 0; i < nargs - 1; i++) {
-				zval_ptr_dtor(&args[i]);
-			}
-			efree(args);
-		}
-		return;
-	}
-	ZVAL_STRING(&handler, (char *) obj->stringval);
-	xmlXPathFreeObject(obj);
-
-	ZVAL_COPY_VALUE(&fci.function_name, &handler);
-	fci.object = NULL;
-	fci.retval = &retval;
-	if (!zend_make_callable(&handler, &callable)) {
-		if (!EG(exception)) {
-			php_error_docref(NULL, E_WARNING, "Unable to call handler %s()", ZSTR_VAL(callable));
-		}
-		valuePush(ctxt, xmlXPathNewString((const xmlChar *) ""));
-	} else if ( intern->registerPhpFunctions == 2 && zend_hash_exists(intern->registered_phpfunctions, callable) == 0) {
-		php_error_docref(NULL, E_WARNING, "Not allowed to call handler '%s()'", ZSTR_VAL(callable));
-		/* Push an empty string, so that we at least have an xslt result... */
-		valuePush(ctxt, xmlXPathNewString((const xmlChar *) ""));
-	} else {
-		zend_call_function(&fci, NULL);
-		if (Z_ISUNDEF(retval)) {
-			/* Exception thrown, don't do anything further. */
-		} else if (Z_TYPE(retval) == IS_OBJECT && instanceof_function(Z_OBJCE(retval), dom_node_class_entry)) {
-			xmlNode *nodep;
-			dom_object *obj;
-			if (intern->node_list == NULL) {
-				intern->node_list = zend_new_array(0);
-			}
-			Z_ADDREF(retval);
-			zend_hash_next_index_insert(intern->node_list, &retval);
-			obj = Z_DOMOBJ_P(&retval);
-			nodep = dom_object_get_node(obj);
-			valuePush(ctxt, xmlXPathNewNodeSet(nodep));
-		} else if (Z_TYPE(retval) == IS_TRUE || Z_TYPE(retval) == IS_FALSE) {
-			valuePush(ctxt, xmlXPathNewBoolean(Z_TYPE(retval) == IS_TRUE));
-		} else if (Z_TYPE(retval) == IS_OBJECT) {
-			php_error_docref(NULL, E_WARNING, "A PHP Object cannot be converted to a XPath-string");
-			valuePush(ctxt, xmlXPathNewString((const xmlChar *) ""));
-		} else {
-			convert_to_string(&retval);
-			valuePush(ctxt, xmlXPathNewString((xmlChar *) Z_STRVAL(retval)));
-		}
-		zval_ptr_dtor(&retval);
-	}
-	zend_string_release_ex(callable, 0);
-	zval_ptr_dtor_nogc(&handler);
-	if (fci.param_count > 0) {
-		for (i = 0; i < nargs - 1; i++) {
-			zval_ptr_dtor(&args[i]);
-		}
-		efree(args);
+		php_dom_xpath_callbacks_call_php_ns(&intern->xpath_callbacks, ctxt, nargs, evaluation_mode, (dom_object *) intern->doc, xsl_proxy_factory);
 	}
 }
 /* }}} */
 
 void xsl_ext_function_string_php(xmlXPathParserContextPtr ctxt, int nargs) /* {{{ */
 {
-	xsl_ext_function_php(ctxt, nargs, 1);
+	xsl_ext_function_php(ctxt, nargs, PHP_DOM_XPATH_EVALUATE_NODESET_TO_STRING);
 }
 /* }}} */
 
 void xsl_ext_function_object_php(xmlXPathParserContextPtr ctxt, int nargs) /* {{{ */
 {
-	xsl_ext_function_php(ctxt, nargs, 2);
+	xsl_ext_function_php(ctxt, nargs, PHP_DOM_XPATH_EVALUATE_NODESET_TO_NODESET);
 }
 /* }}} */
 
+static void xsl_ext_function_trampoline(xmlXPathParserContextPtr ctxt, int nargs)
+{
+	xsl_object *intern = xsl_ext_fetch_intern(ctxt);
+	if (!intern) {
+		php_dom_xpath_callbacks_clean_argument_stack(ctxt, nargs);
+	} else {
+		php_dom_xpath_callbacks_call_custom_ns(&intern->xpath_callbacks, ctxt, nargs, PHP_DOM_XPATH_EVALUATE_NODESET_TO_NODESET, (dom_object *) intern->doc, xsl_proxy_factory);
+	}
+}
+
+static void xsl_add_ns_to_map(xmlHashTablePtr table, xsltStylesheetPtr sheet, const xmlNode *cur, const xmlChar *prefix, const xmlChar *uri)
+{
+	const xmlChar *existing_url = xmlHashLookup(table, prefix);
+	if (existing_url != NULL && !xmlStrEqual(existing_url, uri)) {
+		xsltTransformError(NULL, sheet, (xmlNodePtr) cur, "Namespaces prefix %s used for multiple namespaces\n", prefix);
+		sheet->warnings++;
+	} else if (existing_url == NULL) {
+		xmlHashUpdateEntry(table, prefix, (void *) uri, NULL);
+	}
+}
+
+/* Adds all namespace declaration (not using nsDef) into a hash map that maps prefix to uri. Warns on conflicting declarations. */
+static void xsl_build_ns_map(xmlHashTablePtr table, xsltStylesheetPtr sheet, php_dom_libxml_ns_mapper *ns_mapper, const xmlDoc *doc)
+{
+	const xmlNode *cur = xmlDocGetRootElement(doc);
+
+	while (cur != NULL) {
+		if (cur->type == XML_ELEMENT_NODE) {
+			if (cur->ns != NULL && cur->ns->prefix != NULL) {
+				xsl_add_ns_to_map(table, sheet, cur, cur->ns->prefix, cur->ns->href);
+			}
+
+			for (const xmlAttr *attr = cur->properties; attr != NULL; attr = attr->next) {
+				if (attr->ns != NULL && attr->ns->prefix != NULL && php_dom_ns_is_fast_ex(attr->ns, php_dom_ns_is_xmlns_magic_token)
+					&& attr->children != NULL && attr->children->content != NULL) {
+					/* This attribute declares a namespace, get the relevant instance.
+					 * The declared namespace is not the same as the namespace of this attribute (which is xmlns). */
+					const xmlChar *prefix = attr->name;
+					xmlNsPtr ns = php_dom_libxml_ns_mapper_get_ns_raw_strings_nullsafe(ns_mapper, (const char *) prefix, (const char *) attr->children->content);
+					xsl_add_ns_to_map(table, sheet, cur, prefix, ns->href);
+				}
+			}
+		}
+
+		cur = php_dom_next_in_tree_order(cur, (const xmlNode *) doc);
+	}
+}
+
+/* Apply namespace corrections for new DOM */
+typedef enum {
+	XSL_NS_HASH_CORRECTION_NONE = 0,
+	XSL_NS_HASH_CORRECTION_APPLIED = 1,
+	XSL_NS_HASH_CORRECTION_FAILED = 2
+} xsl_ns_hash_correction_status;
+
+static zend_always_inline xsl_ns_hash_correction_status xsl_apply_ns_hash_corrections(xsltStylesheetPtr sheetp, xmlNodePtr nodep, xmlDocPtr doc)
+{
+	if (sheetp->nsHash == NULL) {
+		dom_object *node_intern = php_dom_object_get_data(nodep);
+		if (node_intern != NULL && php_dom_follow_spec_intern(node_intern)) {
+			sheetp->nsHash = xmlHashCreate(10);
+			if (UNEXPECTED(!sheetp->nsHash)) {
+				return XSL_NS_HASH_CORRECTION_FAILED;
+			}
+			xsl_build_ns_map(sheetp->nsHash, sheetp, php_dom_get_ns_mapper(node_intern), doc);
+			return XSL_NS_HASH_CORRECTION_APPLIED;
+		}
+	}
+	return XSL_NS_HASH_CORRECTION_NONE;
+}
+
 /* {{{ URL: http://www.w3.org/TR/2003/WD-DOM-Level-3-Core-20030226/DOM3-Core.html#
 Since:
 */
@@ -323,9 +191,8 @@ PHP_METHOD(XSLTProcessor, importStylesheet)
 {
 	zval *id, *docp = NULL;
 	xmlDoc *doc = NULL, *newdoc = NULL;
-	xsltStylesheetPtr sheetp, oldsheetp;
-	xsl_object *intern;
-	int clone_docu = 0;
+	xsltStylesheetPtr sheetp;
+	bool clone_docu = false;
 	xmlNode *nodep = NULL;
 	zval *cloneDocu, rv;
 	zend_string *member;
@@ -341,7 +208,7 @@ PHP_METHOD(XSLTProcessor, importStylesheet)
 		doc = nodep->doc;
 	}
 	if (doc == NULL) {
-		zend_argument_value_error(1, "must be a valid XML node");
+		zend_argument_type_error(1, "must be a valid XML node");
 		RETURN_THROWS();
 	}
 
@@ -350,10 +217,10 @@ PHP_METHOD(XSLTProcessor, importStylesheet)
 	newdoc = xmlCopyDoc(doc, 1);
 	xmlNodeSetBase((xmlNodePtr) newdoc, (xmlChar *)doc->URL);
 	PHP_LIBXML_SANITIZE_GLOBALS(parse);
-	PHP_LIBXML_IGNORE_DEPRECATIONS_START
+	ZEND_DIAGNOSTIC_IGNORED_START("-Wdeprecated-declarations")
 	xmlSubstituteEntitiesDefault(1);
 	xmlLoadExtDtdDefaultValue = XML_DETECT_IDS | XML_COMPLETE_ATTRS;
-	PHP_LIBXML_IGNORE_DEPRECATIONS_END
+	ZEND_DIAGNOSTIC_IGNORED_END
 
 	sheetp = xsltParseStylesheetDoc(newdoc);
 	PHP_LIBXML_RESTORE_GLOBALS(parse);
@@ -363,45 +230,60 @@ PHP_METHOD(XSLTProcessor, importStylesheet)
 		RETURN_FALSE;
 	}
 
-	intern = Z_XSL_P(id);
+	xsl_object *intern = Z_XSL_P(id);
+
+	xsl_ns_hash_correction_status status = xsl_apply_ns_hash_corrections(sheetp, nodep, doc);
+	if (UNEXPECTED(status == XSL_NS_HASH_CORRECTION_FAILED)) {
+		xsltFreeStylesheet(sheetp);
+		xmlFreeDoc(newdoc);
+		RETURN_FALSE;
+	} else if (status == XSL_NS_HASH_CORRECTION_APPLIED) {
+		/* The namespace mappings need to be kept alive.
+		 * This is stored in the ref obj outside of libxml2, but that means that the sheet won't keep it alive
+		 * unlike with namespaces from old DOM. */
+		if (intern->sheet_ref_obj) {
+			php_libxml_decrement_doc_ref_directly(intern->sheet_ref_obj);
+		}
+		intern->sheet_ref_obj = Z_LIBXML_NODE_P(docp)->document;
+		intern->sheet_ref_obj->refcount++;
+	}
 
 	member = ZSTR_INIT_LITERAL("cloneDocument", 0);
-	cloneDocu = zend_std_read_property(Z_OBJ_P(id), member, BP_VAR_IS, NULL, &rv);
-	if (Z_TYPE_P(cloneDocu) != IS_NULL) {
-		convert_to_long(cloneDocu);
-		clone_docu = Z_LVAL_P(cloneDocu);
-	}
+	cloneDocu = zend_std_read_property(Z_OBJ_P(id), member, BP_VAR_R, NULL, &rv);
+	clone_docu = zend_is_true(cloneDocu);
 	zend_string_release_ex(member, 0);
-	if (clone_docu == 0) {
-		/* check if the stylesheet is using xsl:key, if yes, we have to clone the document _always_ before a transformation */
+	if (!clone_docu) {
+		/* Check if the stylesheet is using xsl:key, if yes, we have to clone the document _always_ before a transformation.
+		 * xsl:key elements may only occur at the top level. Furthermore, all elements at the top level must be in a
+		 * namespace (if not, then the stylesheet is not well-formed and this function will have returned false earlier). */
 		nodep = xmlDocGetRootElement(sheetp->doc);
 		if (nodep && (nodep = nodep->children)) {
 			while (nodep) {
+				ZEND_ASSERT(nodep->ns != NULL);
 				if (nodep->type == XML_ELEMENT_NODE && xmlStrEqual(nodep->name, (const xmlChar *) "key") && xmlStrEqual(nodep->ns->href, XSLT_NAMESPACE)) {
-					intern->hasKeys = 1;
+					intern->hasKeys = true;
 					break;
 				}
 				nodep = nodep->next;
 			}
 		}
 	} else {
-		intern->hasKeys = clone_docu;
+		intern->hasKeys = true;
 	}
 
-	if ((oldsheetp = (xsltStylesheetPtr)intern->ptr)) {
-		/* free wrapper */
-		if (((xsltStylesheetPtr) intern->ptr)->_private != NULL) {
-			((xsltStylesheetPtr) intern->ptr)->_private = NULL;
-		}
-		xsltFreeStylesheet((xsltStylesheetPtr) intern->ptr);
-		intern->ptr = NULL;
-	}
+	xsl_free_sheet(intern);
 
 	php_xsl_set_object(id, sheetp);
 	RETVAL_TRUE;
 }
 /* }}} end XSLTProcessor::importStylesheet */
 
+static void php_xsl_delayed_lib_registration(void *ctxt, const zend_string *ns, const zend_string *name)
+{
+	xsltTransformContextPtr xsl = (xsltTransformContextPtr) ctxt;
+	xsltRegisterExtFunction(xsl, (const xmlChar *) ZSTR_VAL(name), (const xmlChar *) ZSTR_VAL(ns), xsl_ext_function_trampoline);
+}
+
 static xmlDocPtr php_xsl_apply_stylesheet(zval *id, xsl_object *intern, xsltStylesheetPtr style, zval *docp) /* {{{ */
 {
 	xmlDocPtr newdocp = NULL;
@@ -409,13 +291,10 @@ static xmlDocPtr php_xsl_apply_stylesheet(zval *id, xsl_object *intern, xsltStyl
 	xmlNodePtr node = NULL;
 	xsltTransformContextPtr ctxt;
 	php_libxml_node_object *object;
-	char **params = NULL;
-	int clone;
 	zval *doXInclude, rv;
 	zend_string *member;
 	FILE *f;
 	int secPrefsError = 0;
-	int secPrefsValue;
 	xsltSecurityPrefsPtr secPrefs = NULL;
 
 	node = php_libxml_import_node(docp);
@@ -425,7 +304,7 @@ static xmlDocPtr php_xsl_apply_stylesheet(zval *id, xsl_object *intern, xsltStyl
 	}
 
 	if (doc == NULL) {
-		zend_argument_value_error(1, "must be a valid XML node");
+		zend_argument_type_error(1, "must be a valid XML node");
 		return NULL;
 	}
 
@@ -438,23 +317,19 @@ static xmlDocPtr php_xsl_apply_stylesheet(zval *id, xsl_object *intern, xsltStyl
 	}
 
 	if (intern->profiling) {
-		if (php_check_open_basedir(intern->profiling)) {
+		if (php_check_open_basedir(ZSTR_VAL(intern->profiling))) {
 			f = NULL;
 		} else {
-			f = VCWD_FOPEN(intern->profiling, "w");
+			f = VCWD_FOPEN(ZSTR_VAL(intern->profiling), "w");
 		}
 	} else {
 		f = NULL;
 	}
 
-	if (intern->parameter) {
-		params = php_xsl_xslt_make_params(intern->parameter, 0);
-	}
-
 	intern->doc = emalloc(sizeof(php_libxml_node_object));
 	memset(intern->doc, 0, sizeof(php_libxml_node_object));
 
-	if (intern->hasKeys == 1) {
+	if (intern->hasKeys) {
 		doc = xmlCopyDoc(doc, 1);
 	} else {
 		object = Z_LIBXML_NODE_P(docp);
@@ -466,15 +341,27 @@ static xmlDocPtr php_xsl_apply_stylesheet(zval *id, xsl_object *intern, xsltStyl
 	ctxt = xsltNewTransformContext(style, doc);
 	ctxt->_private = (void *) intern;
 
-	member = ZSTR_INIT_LITERAL("doXInclude", 0);
-	doXInclude = zend_std_read_property(Z_OBJ_P(id), member, BP_VAR_IS, NULL, &rv);
-	if (Z_TYPE_P(doXInclude) != IS_NULL) {
-		convert_to_long(doXInclude);
-		ctxt->xinclude = Z_LVAL_P(doXInclude);
+	if (intern->parameter) {
+		zend_result status = php_xsl_xslt_apply_params(ctxt, intern->parameter);
+		if (UNEXPECTED(status != SUCCESS) && EG(exception)) {
+			goto out;
+		}
 	}
+
+	member = ZSTR_INIT_LITERAL("doXInclude", 0);
+	doXInclude = zend_std_read_property(Z_OBJ_P(id), member, BP_VAR_R, NULL, &rv);
+	ctxt->xinclude = zend_is_true(doXInclude);
 	zend_string_release_ex(member, 0);
 
-	secPrefsValue = intern->securityPrefs;
+	zval *max_template_depth = xsl_prop_max_template_depth(Z_OBJ_P(id));
+	ZEND_ASSERT(Z_TYPE_P(max_template_depth) == IS_LONG);
+	ctxt->maxTemplateDepth = Z_LVAL_P(max_template_depth);
+
+	zval *max_template_vars = xsl_prop_max_template_vars(Z_OBJ_P(id));
+	ZEND_ASSERT(Z_TYPE_P(max_template_vars) == IS_LONG);
+	ctxt->maxTemplateVars = Z_LVAL_P(max_template_vars);
+
+	zend_long secPrefsValue = intern->securityPrefs;
 
 	/* if securityPrefs is set to NONE, we don't have to do any checks, but otherwise... */
 	if (secPrefsValue != XSL_SECPREF_NONE) {
@@ -510,11 +397,15 @@ static xmlDocPtr php_xsl_apply_stylesheet(zval *id, xsl_object *intern, xsltStyl
 		}
 	}
 
+	php_dom_xpath_callbacks_delayed_lib_registration(&intern->xpath_callbacks, ctxt, php_xsl_delayed_lib_registration);
+
 	if (secPrefsError == 1) {
 		php_error_docref(NULL, E_WARNING, "Can't set libxslt security properties, not doing transformation for security reasons");
 	} else {
-		newdocp = xsltApplyStylesheetUser(style, doc, (const char**) params,  NULL, f, ctxt);
+		newdocp = xsltApplyStylesheetUser(style, doc, /* params (handled manually) */ NULL, /* output */ NULL, f, ctxt);
 	}
+
+out:
 	if (f) {
 		fclose(f);
 	}
@@ -524,24 +415,12 @@ static xmlDocPtr php_xsl_apply_stylesheet(zval *id, xsl_object *intern, xsltStyl
 		xsltFreeSecurityPrefs(secPrefs);
 	}
 
-	if (intern->node_list != NULL) {
-		zend_hash_destroy(intern->node_list);
-		FREE_HASHTABLE(intern->node_list);
-		intern->node_list = NULL;
-	}
+	php_dom_xpath_callbacks_clean_node_list(&intern->xpath_callbacks);
 
 	php_libxml_decrement_doc_ref(intern->doc);
 	efree(intern->doc);
 	intern->doc = NULL;
 
-	if (params) {
-		clone = 0;
-		while(params[clone]) {
-			efree(params[clone++]);
-		}
-		efree(params);
-	}
-
 	return newdocp;
 
 }
@@ -555,14 +434,14 @@ PHP_METHOD(XSLTProcessor, transformToDoc)
 	zval *id, *docp = NULL;
 	xmlDoc *newdocp;
 	xsltStylesheetPtr sheetp;
-	zend_string *ret_class = NULL;
+	zend_class_entry *ret_class = NULL;
 	xsl_object *intern;
 
 	id = ZEND_THIS;
 	intern = Z_XSL_P(id);
 	sheetp = (xsltStylesheetPtr) intern->ptr;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "o|S!", &docp, &ret_class) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "o|C!", &docp, &ret_class) == FAILURE) {
 		RETURN_THROWS();
 	}
 
@@ -571,7 +450,7 @@ PHP_METHOD(XSLTProcessor, transformToDoc)
 	if (newdocp) {
 		if (ret_class) {
 			zend_string *curclass_name;
-			zend_class_entry *curce, *ce;
+			zend_class_entry *curce;
 			php_libxml_node_object *interndoc;
 
 			curce = Z_OBJCE_P(docp);
@@ -580,16 +459,15 @@ PHP_METHOD(XSLTProcessor, transformToDoc)
 				curce = curce->parent;
 			}
 
-			ce = zend_lookup_class(ret_class);
-			if (ce == NULL || !instanceof_function(ce, curce)) {
+			if (!instanceof_function(ret_class, curce)) {
 				xmlFreeDoc(newdocp);
-				zend_argument_type_error(2, "must be a class name compatible with %s, \"%s\" given",
-					ZSTR_VAL(curclass_name), ZSTR_VAL(ret_class)
+				zend_argument_type_error(2, "must be a class name compatible with %s, %s given",
+					ZSTR_VAL(curclass_name), ZSTR_VAL(ret_class->name)
 				);
 				RETURN_THROWS();
 			}
 
-			object_init_ex(return_value, ce);
+			object_init_ex(return_value, ret_class);
 
 			interndoc = Z_LIBXML_NODE_P(return_value);
 			php_libxml_increment_doc_ref(interndoc, newdocp);
@@ -600,7 +478,6 @@ PHP_METHOD(XSLTProcessor, transformToDoc)
 	} else {
 		RETURN_FALSE;
 	}
-
 }
 /* }}} end XSLTProcessor::transformToDoc */
 
@@ -688,7 +565,7 @@ PHP_METHOD(XSLTProcessor, setParameter)
 		Z_PARAM_STRING(namespace, namespace_len)
 		Z_PARAM_ARRAY_HT_OR_STR(array_value, name)
 		Z_PARAM_OPTIONAL
-		Z_PARAM_STR_OR_NULL(value)
+		Z_PARAM_PATH_STR_OR_NULL(value)
 	ZEND_PARSE_PARAMETERS_END();
 
 	intern = Z_XSL_P(id);
@@ -707,10 +584,23 @@ PHP_METHOD(XSLTProcessor, setParameter)
 				zend_argument_type_error(2, "must contain only string keys");
 				RETURN_THROWS();
 			}
+
+			if (UNEXPECTED(CHECK_NULL_PATH(ZSTR_VAL(string_key), ZSTR_LEN(string_key)))) {
+				zend_argument_value_error(3, "must not contain keys with any null bytes");
+				RETURN_THROWS();
+			}
+
 			str = zval_try_get_string(entry);
 			if (UNEXPECTED(!str)) {
 				RETURN_THROWS();
 			}
+
+			if (UNEXPECTED(CHECK_NULL_PATH(ZSTR_VAL(str), ZSTR_LEN(str)))) {
+				zend_string_release(str);
+				zend_argument_value_error(3, "must not contain values with any null bytes");
+				RETURN_THROWS();
+			}
+
 			ZVAL_STR(&tmp, str);
 			zend_hash_update(intern->parameter, string_key, &tmp);
 		} ZEND_HASH_FOREACH_END();
@@ -721,6 +611,11 @@ PHP_METHOD(XSLTProcessor, setParameter)
 			RETURN_THROWS();
 		}
 
+		if (UNEXPECTED(CHECK_NULL_PATH(ZSTR_VAL(name), ZSTR_LEN(name)))) {
+			zend_argument_value_error(2, "must not contain any null bytes");
+			RETURN_THROWS();
+		}
+
 		ZVAL_STR_COPY(&new_string, value);
 
 		zend_hash_update(intern->parameter, name, &new_string);
@@ -744,7 +639,7 @@ PHP_METHOD(XSLTProcessor, getParameter)
 	}
 	intern = Z_XSL_P(id);
 	if ((value = zend_hash_find(intern->parameter, name)) != NULL) {
-		RETURN_STR(zval_get_string(value));
+		RETURN_STR_COPY(Z_STR_P(value));
 	} else {
 		RETURN_FALSE;
 	}
@@ -775,59 +670,78 @@ PHP_METHOD(XSLTProcessor, removeParameter)
 /* {{{ */
 PHP_METHOD(XSLTProcessor, registerPHPFunctions)
 {
-	zval *id = ZEND_THIS;
-	xsl_object *intern;
-	zval *entry, new_string;
-	zend_string *restrict_str = NULL;
-	HashTable *restrict_ht = NULL;
+	xsl_object *intern = Z_XSL_P(ZEND_THIS);
+
+	zend_string *name = NULL;
+	HashTable *callable_ht = NULL;
 
 	ZEND_PARSE_PARAMETERS_START(0, 1)
 		Z_PARAM_OPTIONAL
-		Z_PARAM_ARRAY_HT_OR_STR_OR_NULL(restrict_ht, restrict_str)
+		Z_PARAM_ARRAY_HT_OR_STR_OR_NULL(callable_ht, name)
 	ZEND_PARSE_PARAMETERS_END();
 
-	intern = Z_XSL_P(id);
-
-	if (restrict_ht) {
-		ZEND_HASH_FOREACH_VAL(restrict_ht, entry) {
-			zend_string *str = zval_try_get_string(entry);
-			if (UNEXPECTED(!str)) {
-				return;
-			}
-			ZVAL_LONG(&new_string, 1);
-			zend_hash_update(intern->registered_phpfunctions, str, &new_string);
-			zend_string_release(str);
-		} ZEND_HASH_FOREACH_END();
-
-		intern->registerPhpFunctions = 2;
-	} else if (restrict_str) {
-		ZVAL_LONG(&new_string, 1);
-		zend_hash_update(intern->registered_phpfunctions, restrict_str, &new_string);
-		intern->registerPhpFunctions = 2;
-	} else {
-		intern->registerPhpFunctions = 1;
-	}
+	php_dom_xpath_callbacks_update_method_handler(
+		&intern->xpath_callbacks,
+		NULL,
+		NULL,
+		name,
+		callable_ht,
+		PHP_DOM_XPATH_CALLBACK_NAME_VALIDATE_NULLS,
+		NULL
+	);
 }
 /* }}} end XSLTProcessor::registerPHPFunctions(); */
 
+PHP_METHOD(XSLTProcessor, registerPHPFunctionNS)
+{
+	xsl_object *intern = Z_XSL_P(ZEND_THIS);
+
+	zend_string *namespace, *name;
+	zend_fcall_info fci;
+	zend_fcall_info_cache fcc;
+
+	ZEND_PARSE_PARAMETERS_START(3, 3)
+		Z_PARAM_PATH_STR(namespace)
+		Z_PARAM_PATH_STR(name)
+		Z_PARAM_FUNC_NO_TRAMPOLINE_FREE(fci, fcc)
+	ZEND_PARSE_PARAMETERS_END();
+
+	if (zend_string_equals_literal(namespace, "http://php.net/xsl")) {
+		zend_release_fcall_info_cache(&fcc);
+		zend_argument_value_error(1, "must not be \"http://php.net/xsl\" because it is reserved by PHP");
+		RETURN_THROWS();
+	}
+
+	if (php_dom_xpath_callbacks_update_single_method_handler(
+		&intern->xpath_callbacks,
+		NULL,
+		namespace,
+		name,
+		&fcc,
+		PHP_DOM_XPATH_CALLBACK_NAME_VALIDATE_NCNAME,
+		NULL
+	) != SUCCESS) {
+		zend_release_fcall_info_cache(&fcc);
+	}
+}
+
 /* {{{ */
 PHP_METHOD(XSLTProcessor, setProfiling)
 {
 	zval *id = ZEND_THIS;
 	xsl_object *intern;
-	char *filename = NULL;
-	size_t filename_len;
+	zend_string *filename = NULL;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "p!", &filename, &filename_len) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "P!", &filename) == FAILURE) {
 		RETURN_THROWS();
 	}
 
 	intern = Z_XSL_P(id);
 	if (intern->profiling) {
-		efree(intern->profiling);
+		zend_string_release(intern->profiling);
 	}
 	if (filename != NULL) {
-		intern->profiling = estrndup(filename, filename_len);
+		intern->profiling = zend_string_copy(filename);
 	} else {
 		intern->profiling = NULL;
 	}
@@ -849,8 +763,6 @@ PHP_METHOD(XSLTProcessor, setSecurityPrefs)
 	intern = Z_XSL_P(id);
 	oldSecurityPrefs = intern->securityPrefs;
 	intern->securityPrefs = securityPrefs;
-	/* set this to 1 so that we know, it was set through this method. Can be removed, when we remove the ini setting */
-	intern->securityPrefsSet = 1;
 	RETURN_LONG(oldSecurityPrefs);
 }
 /* }}} end XSLTProcessor::setSecurityPrefs */
