@@ -1,13 +1,11 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
-   +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2017 The PHP Group                                |
+   | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_01.txt                                  |
+   | https://www.php.net/license/3_01.txt                                 |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -23,8 +21,10 @@
 
 #include "TSRM.h"
 
+zend_string *phpdbg_compile_stackframe(zend_execute_data *);
 void phpdbg_restore_frame(void);
 void phpdbg_switch_frame(int);
 void phpdbg_dump_backtrace(size_t);
+void phpdbg_open_generator_frame(zend_generator *);
 
 #endif /* PHPDBG_FRAME_H */
