@@ -3,17 +3,14 @@
 
 #include <signal.h>
 
+#include "win32/winutil.h"
+
 #define SIGALRM 13
 #define	SIGVTALRM 26			/* virtual time alarm */
 #define	SIGPROF	27				/* profiling time alarm */
 
-#endif /* PHP_WIN32_SIGNAL_H */
+PHP_WINUTIL_API void php_win32_signal_ctrl_handler_init(void);
+PHP_WINUTIL_API void php_win32_signal_ctrl_handler_request_shutdown(void);
+PHP_WINUTIL_API void php_win32_signal_ctrl_handler_shutdown(void);
 
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: sw=4 ts=4 fdm=marker
- * vim<600: sw=4 ts=4
- */
+#endif /* PHP_WIN32_SIGNAL_H */
